@@ -32,7 +32,7 @@ package Apache2::AMFDetectRightFilter;
   # 
 
   use vars qw($VERSION);
-  $VERSION= "3.54";
+  $VERSION= "4.00a";
   my $CommonLib = new Apache2::AMFCommonLib ();
  
   my %Capability;
@@ -252,8 +252,13 @@ sub loadConfigFile {
 				my @rows = split(/\n/, $content);
 				my $row;
 				my $count=0;
+                              my $numberRow=scalar(@rows);
+                              my $progress=0;                                                      
 				foreach $row (@rows){
 					$r_id=parseDetectRightFile($row,$r_id);
+                                        $progress++;
+                                        my $perc=int(($progress/$numberRow)*100);
+                                        print "Percent loaded: ".$perc."%\r";                                
 				}
 			}
 			$CommonLib->printLog("Finish downloading DetectRight from $downloadDetectRighturl");
@@ -269,8 +274,13 @@ sub loadConfigFile {
 						$string_file =~ s/\n//g;
 						$string_file =~ s/>/>\n/g;
 						my @arrayFile=split(/\n/, $string_file);
+                                                my $numberRow=scalar(@arrayFile);
+                                                my $progress=0;                                                      
 						foreach my $line (@arrayFile) {
-							$r_id=parseDetectRightFile($line,$r_id);
+	     					    $r_id=parseDetectRightFile($line,$r_id);
+                                                    $progress++;
+                                                    my $perc=int(($progress/$numberRow)*100);
+                                                    print "Percent loaded: ".$perc."%\r";                                
 						}
 					} else {
 					    $CommonLib->printLog("Error open file:$fileDetectRight");
